/**
 * Copyright or © or Copr. IETR/INSA - Rennes (2018 - 2019) :
 *
 * Antoine Morvan <antoine.morvan@insa-rennes.fr> (2018 - 2019)
 *
 * This software is a computer program whose purpose is to help prototyping
 * parallel applications using dataflow formalism.
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * @file main.c
 * @generated by $PREESM_PRINTER
 * @date $PREESM_DATE
 */

#[[#]]#include "preesm_gen_tcp.h"

#[[#]]#include <execinfo.h>
#[[#]]#include <signal.h>

#[[#]]#define PREESM_COM_PORT 25400
#[[#]]#define _PREESM_NBTHREADS_ $PREESM_NBTHREADS
#[[#]]#define _PREESM_MAIN_THREAD_ $PREESM_MAIN_THREAD

ProcessingElement registry[_PREESM_NBTHREADS_];

$PREESM_THREAD_FUNCTIONS_DECLS

void *(*coreThreadComputations[_PREESM_NBTHREADS_])(void *) = {
  $PREESM_THREAD_FUNCTIONS
};


void handler(int sig) {
  void *array[30];
  size_t size;
  size = backtrace(array, 30);
  fprintf(stderr, "Error: signal %d:\n", sig);
  backtrace_symbols_fd(array, size, STDERR_FILENO);
  exit(1);
}

// kept for compatibility
int preesmStopThreads;

void actualThreadComputations(int processingElementID) {
  registry[processingElementID].id = processingElementID;
  int socketFileDescriptors[_PREESM_NBTHREADS_ + 1];
  socketFileDescriptors[_PREESM_NBTHREADS_] = processingElementID;

  preesm_open(socketFileDescriptors, processingElementID, _PREESM_NBTHREADS_, registry);
#[[#]]#ifdef _PREESM_TCP_DEBUG_
  printf("[TCP-DEBUG] %d READY\n", processingElementID);
#[[#]]#endif
  coreThreadComputations[processingElementID](socketFileDescriptors);
#[[#]]#ifdef _PREESM_TCP_DEBUG_
  printf("[TCP-DEBUG] %d DONE\n", processingElementID);
#[[#]]#endif
  preesm_close(socketFileDescriptors, processingElementID, _PREESM_NBTHREADS_);
}

void * threadComputations (void *arg) {
  int processingElementID = *((int*) arg);
  actualThreadComputations(processingElementID);
  return NULL;
}


void initMainPEConfig(ProcessingElement registry[_PREESM_NBTHREADS_]) {
  char * buffer = 0;
  long length;
  FILE * f = fopen ("socketcom.conf", "r");
  if (f) {
    fseek (f, 0, SEEK_END);
    length = ftell (f);
    fseek (f, 0, SEEK_SET);
    buffer = malloc (length);
    if (buffer) {
      int res = fread (buffer, 1, length, f);
      if (res <= 0) {
        printf("Could not read config file");
        return;
      }
    }
    fclose (f);
    char *r = buffer;
    char *tok = r, *end = r;
    strsep(&end, ":");
    char* host = strdup(tok);
    tok = end;
    strsep(&end, ":");
    char* portStr  = tok;
    int port = atoi(portStr);
    free(r);
    strcpy(registry[0].host,host);
    registry[0].port = port;
    registry[0].id = 0;
  } else {
    registry[0].port = 25400;
    registry[0].id = 0;
    strcpy(registry[0].host,"127.0.0.1");
    printf("Warning: Could not locate file 'socketcom.conf'. Using '127.0.0.1:25400'.\n");
  }
}

int main(int argc, char** argv) {
  signal(SIGSEGV, handler);
  signal(SIGPIPE, handler);

  initMainPEConfig(registry);

  if (argc == 2) {
    // read ID from arguments
    int peId = atoi(argv[1]);
#[[#]]#ifdef _PREESM_TCP_DEBUG_
  printf("[TCP-DEBUG] Running %d only\n", peId);
#[[#]]#endif
    actualThreadComputations(peId);
  } else {
    // launch all IDs in separate threads
    pthread_t threads[_PREESM_NBTHREADS_];
    int threadArguments[_PREESM_NBTHREADS_];
    for (int i = 0; i < _PREESM_NBTHREADS_; i++) {
      if (i != _PREESM_MAIN_THREAD_) {
        threadArguments[i] = i;
#[[#]]#ifdef _PREESM_TCP_DEBUG_
        printf("[TCP-DEBUG] Launching %d\n", i);
#[[#]]#endif
        pthread_create(&threads[i], NULL, threadComputations, &(threadArguments[i]));
      }
    }
    threadArguments[_PREESM_MAIN_THREAD_] = _PREESM_MAIN_THREAD_;
    threadComputations(&(threadArguments[_PREESM_MAIN_THREAD_]));
    for (int i = 0; i < _PREESM_NBTHREADS_; i++) {
      if (i != _PREESM_MAIN_THREAD_) {
        pthread_join(threads[i], NULL);
      }
    }
  }
  return 0;
}



